/// <reference types="node" />
import React from 'react';
import type { WithNullableEventHandlers } from '../../types';
export declare type Option = {
    value: string;
    id: string;
    className?: string;
};
declare type RenderOptions = {
    dataUi?: string;
    index: number;
    name: string;
    option: Option;
    selected: boolean;
    value: string;
    onOptionRequest?: (value: Option, e: React.MouseEvent) => void;
};
declare type Props = WithNullableEventHandlers<{
    className?: string;
    'data-ui'?: string;
    error: boolean | {};
    options: Array<Option>;
    value: string;
    onBlur: (e: Event) => void;
    onDatalistChangeRequest: (value: Option | string, e: Event) => void;
} & Partial<DefaultProps> & Omit<React.InputHTMLAttributes<HTMLInputElement>, keyof DefaultProps | 'value' | 'onBlur'>>;
declare type DefaultProps = {
    disabled: boolean;
    name: string;
    size: 'small' | '';
    visibleDatalist: boolean;
    stopPropagation: boolean;
    exact: boolean;
    emptyStateVisible: boolean;
    emptyStateValue: string;
    getDatalistValue: (option: Option | string) => string;
    renderOption: (arg0: RenderOptions) => any;
    renderEmptyOption: (arg0: {
        value: string;
    }) => any;
};
declare type State = {
    activeOption: Option;
    visibleDatalist: boolean;
    datalistSelected: number;
    highlightedItemIndex: number;
    options: Array<Option>;
    left: number;
    top: number;
    width: number;
};
export default class ComboBox extends React.PureComponent<Props, State> {
    _scrolling: boolean;
    _timeout: NodeJS.Timer | null | undefined;
    _comboBoxContainer: HTMLDivElement | null | undefined;
    _comboBoxInput: HTMLInputElement | null | undefined;
    static displayName: string;
    static defaultProps: DefaultProps;
    state: State;
    componentDidUpdate(): void;
    _scrolling: boolean;
    _timeout: NodeJS.Timer;
    _findOption: (options: Array<Option>, value: string) => Option;
    _findOptionIndex: (options: Array<Option>, activeOption: Option) => number;
    _handleFocus: () => void;
    _handleBlur: (e: Event) => void;
    _handleDatalistChangeRequest: (selectedOption: Option | string, e: any) => void;
    _showDataList: () => void;
    _hideDataList: () => void;
    _handleToggleDatalist: () => void;
    _handleOptionRequest: (value: Option, e: any) => void;
    _handleComboBoxInput: () => void;
    _deltaIndexSelectedChange: (delta: number) => void;
    _isDatalistVisible: () => boolean;
    _handleComboBoxKeyDown: (e: React.KeyboardEvent<HTMLInputElement>) => void;
    _scrollIntoView: (node: HTMLDivElement) => void;
    setContainerPosition(): void;
    _handleWindowResize: () => void;
    _shouldRenderEmptyState(): boolean;
    render(): JSX.Element;
}
export {};
