"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var _react = _interopRequireDefault(require("react"));

var _classnames = _interopRequireDefault(require("classnames"));

var _escapeStringRegexp = _interopRequireDefault(require("escape-string-regexp"));

var _reactWindowResizeListener = require("react-window-resize-listener");

var _comboBoxOption = _interopRequireDefault(require("./combo-box-option"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var KEYS = {
  DOWN: 40,
  UP: 38,
  ENTER: 13,
  ESC: 27,
  TAB: 9
};

var normalizeValue = function normalizeValue(value) {
  return value.replace(/\s/g, '').toLowerCase();
};

var ComboBox = /*#__PURE__*/function (_React$PureComponent) {
  _inheritsLoose(ComboBox, _React$PureComponent);

  function ComboBox() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$PureComponent.call.apply(_React$PureComponent, [this].concat(args)) || this;

    _defineProperty(_assertThisInitialized(_this), "state", {
      visibleDatalist: _this.props.visibleDatalist,
      datalistSelected: -1,
      activeOption: {
        value: _this.props.value || '',
        id: ''
      },
      highlightedItemIndex: -1,
      options: _this.props.options,
      left: 0,
      top: 0,
      width: 0
    });

    _defineProperty(_assertThisInitialized(_this), "_scrolling", false);

    _defineProperty(_assertThisInitialized(_this), "_timeout", null);

    _defineProperty(_assertThisInitialized(_this), "_findOption", function (options, value) {
      if (options === void 0) {
        options = [];
      }

      return options.find(function (option) {
        return normalizeValue(_this.props.getDatalistValue(option)) === normalizeValue(value);
      });
    });

    _defineProperty(_assertThisInitialized(_this), "_findOptionIndex", function (options, activeOption) {
      if (options === void 0) {
        options = [];
      }

      if (!activeOption) {
        return -1;
      }

      return options.findIndex(function (option) {
        return option.id === activeOption.id;
      });
    });

    _defineProperty(_assertThisInitialized(_this), "_handleFocus", function () {
      var inputEl = _this._comboBoxInput;

      if (!inputEl) {
        return;
      }

      inputEl.focus();
      inputEl.select();
    });

    _defineProperty(_assertThisInitialized(_this), "_handleBlur", function (e) {
      if (_this.state.visibleDatalist) {
        var _value = _this._comboBoxInput ? _this._comboBoxInput.value : '';

        _this._handleDatalistChangeRequest(_value, e);
      }

      if (_this.props.onBlur) {
        _this.props.onBlur(e);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_handleDatalistChangeRequest", function (selectedOption, e) {
      if (selectedOption === void 0) {
        selectedOption = '';
      }

      if (_this.props.onDatalistChangeRequest) {
        var effectiveOption = selectedOption;

        if (_this.props.exact) {
          var dataListValue = _this.props.getDatalistValue(effectiveOption);

          var _option = _this._findOption(_this.props.options, dataListValue);

          if (_option) {
            effectiveOption = typeof effectiveOption === 'object' ? effectiveOption : _option;
          } else {
            effectiveOption = _this.state.activeOption;
          }
        }

        _this.props.onDatalistChangeRequest(effectiveOption, e);

        if (typeof effectiveOption === 'object') {
          _this.setState({
            activeOption: effectiveOption
          });
        }
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_showDataList", function () {
      _this.setState({
        options: _this.props.options,
        visibleDatalist: true,
        datalistSelected: _this._findOptionIndex(_this.props.options, _this.state.activeOption)
      });
    });

    _defineProperty(_assertThisInitialized(_this), "_hideDataList", function () {
      _this.setState({
        visibleDatalist: false
      });
    });

    _defineProperty(_assertThisInitialized(_this), "_handleToggleDatalist", function () {
      _this._handleFocus();

      _this.setState({
        options: _this.props.options,
        visibleDatalist: !_this._isDatalistVisible(),
        highlightedItemIndex: -1,
        datalistSelected: _this._findOptionIndex(_this.props.options, _this.state.activeOption)
      });
    });

    _defineProperty(_assertThisInitialized(_this), "_handleOptionRequest", function (value, e) {
      _this._handleToggleDatalist();

      _this._handleDatalistChangeRequest(value, e);
    });

    _defineProperty(_assertThisInitialized(_this), "_handleComboBoxInput", function () {
      if (_this._comboBoxInput) {
        var inputValue = _this._comboBoxInput.value;
        var normalizedValue = normalizeValue((0, _escapeStringRegexp["default"])(inputValue));
        var regexp = new RegExp("" + normalizedValue, 'i');

        var filteredOptions = _this.props.options.filter(function (option) {
          var value = normalizeValue(_this.props.getDatalistValue(option));
          return regexp.test(value);
        });

        _this.setState({
          options: filteredOptions,
          visibleDatalist: true,
          datalistSelected: filteredOptions.length ? 0 : -1
        });
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_deltaIndexSelectedChange", function (delta) {
      var nextIndex = _this.state.datalistSelected + delta;

      if (nextIndex < 0 || _this.state.datalistSelected === null) {
        nextIndex = 0;
      } else if (nextIndex >= _this.props.options.length) {
        nextIndex = _this.props.options.length - 1;
      }

      _this.setState({
        datalistSelected: nextIndex
      });
    });

    _defineProperty(_assertThisInitialized(_this), "_isDatalistVisible", function () {
      return _this.state.visibleDatalist;
    });

    _defineProperty(_assertThisInitialized(_this), "_handleComboBoxKeyDown", function (e) {
      var key = e.keyCode;

      if (_this.props.stopPropagation) {
        e.stopPropagation();
      } // NOTE: Key down


      if (key === KEYS.DOWN) {
        if (_this._isDatalistVisible()) {
          _this._deltaIndexSelectedChange(1);
        } else {
          _this._handleToggleDatalist();
        }

        e.preventDefault(); // NOTE: Key up
      } else if (key === KEYS.UP) {
        if (_this._isDatalistVisible()) {
          _this._deltaIndexSelectedChange(-1);

          e.preventDefault();
        } // NOTE: Keys escape or tab

      } else if (key === KEYS.ESC) {
        if (_this._isDatalistVisible()) {
          _this._hideDataList();
        } // NOTE: Key enter

      } else if (key === KEYS.TAB) {
        if (_this._isDatalistVisible() && _this.state.datalistSelected >= 0) {
          _this._handleOptionRequest(_this.state.options[_this.state.datalistSelected], e);

          return;
        }

        _this._hideDataList();
      } else if (key === KEYS.ENTER) {
        if (_this._isDatalistVisible()) {
          e.preventDefault();

          if (_this.state.datalistSelected >= 0) {
            _this._handleOptionRequest(_this.state.options[_this.state.datalistSelected], e);

            return;
          }

          if (_this.state.highlightedItemIndex >= 0) {
            _this._handleOptionRequest(_this.state.options[_this.state.highlightedItemIndex], e);
          }
        } else {
          var _value2 = _this._comboBoxInput ? _this._comboBoxInput.value : '';

          _this._handleDatalistChangeRequest(_value2, e);
        }
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_scrollIntoView", function (node) {
      if ('scrollIntoViewIfNeeded' in node) {
        if (_this._timeout) {
          clearTimeout(_this._timeout);
        }

        _this._scrolling = true; // @ts-ignore scrollIntoViewIfNeeded is not supported

        node.scrollIntoViewIfNeeded(); // NOTE: dirty hack to block mouse events

        _this._timeout = setTimeout(function () {
          _this._scrolling = false;
        }, 100);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_handleWindowResize", function () {
      _this.setContainerPosition();
    });

    return _this;
  }

  var _proto = ComboBox.prototype;

  _proto.componentDidUpdate = function componentDidUpdate() {
    if (this._isDatalistVisible()) {
      if (document.body) {
        document.body.classList.add('root--combo-box-visible');
      }

      this.setContainerPosition();
    } else if (document.body) {
      document.body.classList.remove('root--combo-box-visible');
    }
  };

  _proto.setContainerPosition = function setContainerPosition() {
    if (!this._comboBoxContainer) {
      return;
    }

    var containerRect = this._comboBoxContainer.getBoundingClientRect();

    this.setState({
      width: containerRect.width,
      left: containerRect.left,
      top: containerRect.bottom
    });
  };

  _proto._shouldRenderEmptyState = function _shouldRenderEmptyState() {
    return this.props.emptyStateVisible && this.state.options.length === 0;
  };

  _proto.render = function render() {
    var _this2 = this,
        _classNames,
        _classNames2,
        _classNames3,
        _classNames4;

    var inputProps = _extends({}, this.props);

    delete inputProps.options;
    delete inputProps.visibleDatalist;
    delete inputProps.stopPropagation;
    delete inputProps.error;
    delete inputProps.onDatalistChangeRequest;
    delete inputProps.getDatalistValue;
    delete inputProps.renderOption;
    delete inputProps.exact;
    delete inputProps.onBlur;
    delete inputProps.emptyStateVisible;
    delete inputProps.emptyStateValue;
    delete inputProps.renderEmptyOption;
    return /*#__PURE__*/_react["default"].createElement("div", {
      ref: function ref(node) {
        _this2._comboBoxContainer = node;
      },
      className: (0, _classnames["default"])((_classNames = {
        'combo-box__container': true,
        'combo-box__container--disabled': this.props.disabled
      }, _classNames["combo-box__container--" + this.props.name] = Boolean(this.props.name), _classNames))
    }, /*#__PURE__*/_react["default"].createElement(_reactWindowResizeListener.WindowResizeListener, {
      onResize: this._handleWindowResize
    }), /*#__PURE__*/_react["default"].createElement("div", {
      className: (0, _classnames["default"])({
        'combo-box__input-container': true,
        'combo-box__input-container--with-datalist': this._isDatalistVisible(),
        'combo-box__input-container--no-datalist': !this.props.emptyStateVisible && this.state.options.length === 0
      })
    }, /*#__PURE__*/_react["default"].createElement("input", _extends({}, inputProps, {
      onKeyDown: this._handleComboBoxKeyDown,
      ref: function ref(node) {
        _this2._comboBoxInput = node;
      },
      className: (0, _classnames["default"])(this.props.className, (_classNames2 = {
        'combo-box': true
      }, _classNames2["combo-box--" + this.props.name] = Boolean(this.props.name), _classNames2["combo-box--" + this.props.size] = Boolean(this.props.size), _classNames2['combo-box--error'] = this.props.error ? true : undefined, _classNames2)),
      onInput: this._handleComboBoxInput,
      onFocus: this._handleFocus,
      onBlur: this._handleBlur,
      onClick: this._showDataList
    })), /*#__PURE__*/_react["default"].createElement("div", {
      "data-ui": inputProps['data-ui'] && inputProps['data-ui'] + "--trigger",
      className: (0, _classnames["default"])((_classNames3 = {
        'combo-box__trigger': true,
        'combo-box__trigger--datalist-open': this._isDatalistVisible()
      }, _classNames3["combo-box__trigger--" + this.props.name] = Boolean(this.props.name), _classNames3["combo-box__trigger--" + this.props.size] = Boolean(this.props.size), _classNames3)),
      onClick: this._handleToggleDatalist
    })), this._isDatalistVisible() && /*#__PURE__*/_react["default"].createElement("div", {
      "data-ui": inputProps['data-ui'] && inputProps['data-ui'] + "--datalist-container",
      className: (0, _classnames["default"])({
        'combo-box__datalist-container': true,
        'combo-box__datalist-container--open': this._isDatalistVisible()
      })
    }, /*#__PURE__*/_react["default"].createElement("div", {
      className: "combo-box__backdrop",
      onClick: this._hideDataList
    }), /*#__PURE__*/_react["default"].createElement("div", {
      className: (0, _classnames["default"])((_classNames4 = {
        'combo-box__datalist': true,
        'combo-box__datalist--active-selection': this.state.datalistSelected >= 0
      }, _classNames4["combo-box__datalist--" + this.props.name] = Boolean(this.props.name), _classNames4["combo-box__datalist--" + this.props.size] = Boolean(this.props.size), _classNames4)),
      style: {
        left: this.state.left,
        top: this.state.top,
        width: this.state.width
      }
    }, this.state.options.map(function (option, index) {
      var selected = index === _this2.state.datalistSelected;
      return /*#__PURE__*/_react["default"].createElement("div", {
        key: "option-" + index,
        className: "combo-box__option-container-" + index,
        onMouseEnter: function onMouseEnter() {
          if (_this2._scrolling) {
            return;
          }

          _this2.setState({
            datalistSelected: -1,
            highlightedItemIndex: index
          });
        },
        onMouseLeave: function onMouseLeave() {
          _this2.setState({
            datalistSelected: _this2.state.datalistSelected === -1 ? index : _this2.state.datalistSelected,
            highlightedItemIndex: -1
          });
        },
        ref: function ref(node) {
          if (node && selected) {
            _this2._scrollIntoView(node);
          }
        }
      }, _this2.props.renderOption({
        value: _this2.props.getDatalistValue(option),
        option: option,
        index: index,
        selected: selected,
        name: _this2.props.name,
        dataUi: inputProps['data-ui'],
        onOptionRequest: _this2._handleOptionRequest
      }));
    }), this._shouldRenderEmptyState() ? this.props.renderEmptyOption({
      value: this.props.emptyStateValue
    }) : null)));
  };

  return ComboBox;
}(_react["default"].PureComponent);

exports["default"] = ComboBox;

_defineProperty(ComboBox, "displayName", 'ComboBox');

_defineProperty(ComboBox, "defaultProps", {
  disabled: false,
  name: '',
  size: '',
  visibleDatalist: false,
  stopPropagation: true,
  exact: false,
  emptyStateVisible: true,
  emptyStateValue: 'No results found',
  // @ts-ignore
  getDatalistValue: function getDatalistValue(option) {
    return option.value || option;
  },
  renderOption: function renderOption(_ref) {
    var value = _ref.value,
        index = _ref.index,
        name = _ref.name,
        dataUi = _ref.dataUi,
        selected = _ref.selected,
        onOptionRequest = _ref.onOptionRequest,
        option = _ref.option;
    return /*#__PURE__*/_react["default"].createElement(_comboBoxOption["default"], {
      "data-ui": dataUi,
      index: index,
      option: option,
      name: name,
      selected: selected,
      value: value,
      onOptionRequest: onOptionRequest
    });
  },
  renderEmptyOption: function renderEmptyOption(_ref2) {
    var value = _ref2.value;
    return /*#__PURE__*/_react["default"].createElement(_comboBoxOption["default"], {
      value: value,
      disabled: true
    });
  }
});