"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var _react = _interopRequireDefault(require("react"));

var _classnames = _interopRequireDefault(require("classnames"));

var _dropdownList = _interopRequireDefault(require("./dropdown-list"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Dropdown = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(Dropdown, _React$Component);

  function Dropdown() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;

    _defineProperty(_assertThisInitialized(_this), "state", {
      isDropdownVisible: _this.props.visible || false,
      overridenPosition: null,
      dropdownOffset: 0,
      boundingElementDimensions: null,
      dropdownDimensions: null
    });

    _defineProperty(_assertThisInitialized(_this), "_handleDropdown", function () {
      if (_this.props.visible !== null && _this.props.onVisibilityChange) {
        _this.props.onVisibilityChange(!_this.state.isDropdownVisible);
      } else {
        _this.setState({
          isDropdownVisible: !_this.state.isDropdownVisible
        });
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_adjustPosition", function () {
      var dropdown = _this._dropdown;

      if (!_this.props.autoPosition || !dropdown) {
        return;
      }

      var hasBoundingElement = Boolean(_this.props.boundingElement);

      var isHover = _this.props.triggerEvents.includes('hover');

      if (hasBoundingElement && _this.state.dropdownDimensions) {
        return;
      }

      if (isHover && _this._dropdownWrapper) {
        _this._dropdownWrapper.style.display = 'block';
      }

      var boundingElement = _this.props.boundingElement || document.body;

      if (!boundingElement) {
        return;
      }

      var boundingElementDimensions = boundingElement.getBoundingClientRect();
      var dropdownDimensions = dropdown.getBoundingClientRect();

      if (isHover && _this._dropdownWrapper) {
        _this._dropdownWrapper.style.display = '';
      }

      if (_this.state.dropdownDimensions && _this.state.dropdownDimensions.top === dropdownDimensions.top && _this.state.dropdownDimensions.bottom === dropdownDimensions.bottom && _this.state.dropdownDimensions.left === dropdownDimensions.left && _this.state.dropdownDimensions.right === dropdownDimensions.right) {
        return;
      }

      var nextState = {
        boundingElementDimensions: boundingElementDimensions,
        dropdownDimensions: dropdownDimensions
      };

      if (_this.props.position === 'bottom') {
        var shouldMoveUp = hasBoundingElement ? dropdownDimensions.bottom > boundingElementDimensions.bottom : window.innerHeight < dropdownDimensions.bottom;
        nextState = _extends({}, nextState, {
          // @ts-ignore
          overridenPosition: shouldMoveUp ? 'top' : null
        });
      } else if (_this.props.position === 'top') {
        var shouldMoveDown = hasBoundingElement ? dropdownDimensions.top < boundingElementDimensions.top : dropdownDimensions.top < 0;
        nextState = _extends({}, nextState, {
          // @ts-ignore
          overridenPosition: shouldMoveDown ? 'bottom' : null
        });
      }

      var dropdownOffset = 0;

      if (dropdownDimensions.left < boundingElementDimensions.left) {
        dropdownOffset = Math.abs(dropdownDimensions.left - boundingElementDimensions.left);
      } else if (dropdownDimensions.right > boundingElementDimensions.right) {
        dropdownOffset = -(dropdownDimensions.right - boundingElementDimensions.right);
      }

      nextState = _extends({}, nextState, {
        // @ts-ignore
        dropdownOffset: dropdownOffset
      });

      _this.setState(nextState);
    });

    return _this;
  }

  var _proto = Dropdown.prototype;

  _proto.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    if (nextProps.disabled) {
      this.setState({
        isDropdownVisible: false
      });
    }

    if (this.props.visible !== nextProps.visible) {
      this.setState({
        isDropdownVisible: nextProps.visible
      });
    }
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    if (!prevState.isDropdownVisible && this.state.isDropdownVisible) {
      this._adjustPosition();
    }
  };

  _proto._createEventHandlers = function _createEventHandlers() {
    var _this2 = this;

    var eventHandlers = {};

    if (this.props.triggerEvents && Array.isArray(this.props.triggerEvents)) {
      this.props.triggerEvents.forEach(function (event) {
        eventHandlers[event] = _this2._handleDropdown;
      });
    } else {
      // @ts-ignore
      eventHandlers[this.props.triggerEvents] = this._handleDropdown;
    }

    delete eventHandlers['hover'];
    return eventHandlers;
  };

  _proto.render = function render() {
    var _classNames,
        _classNames2,
        _classNames3,
        _classNames4,
        _classNames5,
        _classNames6,
        _classNames7,
        _classNames8,
        _this3 = this;

    var eventHandlers = this._createEventHandlers();

    var dropdown;
    var position = this.state.overridenPosition || this.props.position;
    var hoverClassName = false;

    if (Array.isArray(this.props.triggerEvents)) {
      hoverClassName = this.props.triggerEvents.indexOf('hover') !== -1;
    } else if (this.props.triggerEvents === 'hover') {
      hoverClassName = true;
    }

    var containerClassNames = (0, _classnames["default"])(this.props.containerClassNames, (_classNames = {
      'dropdown-container': true,
      'dropdown-container--legacy': true,
      'dropdown-container--hover': hoverClassName
    }, _classNames["dropdown-container--" + this.props.name] = Boolean(this.props.name), _classNames["dropdown-container--" + this.props.theme] = true, _classNames));
    var dropdownWrapperClassNames = (0, _classnames["default"])((_classNames2 = {
      'dropdown__wrapper': true,
      'dropdown__wrapper--legacy': true
    }, _classNames2["dropdown__wrapper--" + position] = true, _classNames2["dropdown__wrapper--" + this.props.arrowPosition] = true, _classNames2["dropdown__wrapper--" + position + "-" + this.props.arrowPosition] = true, _classNames2['dropdown__wrapper--visible'] = this.state.isDropdownVisible, _classNames2['dropdown__wrapper--hover'] = hoverClassName, _classNames2["dropdown__wrapper--" + this.props.name] = Boolean(this.props.name), _classNames2["dropdown__wrapper--" + this.props.theme] = true, _classNames2));
    var dropdownClassNames = (0, _classnames["default"])(this.props.className, (_classNames3 = {
      'dropdown': true,
      'dropdown--legacy': true
    }, _classNames3["dropdown--" + position] = true, _classNames3['dropdown--visible'] = this.state.isDropdownVisible, _classNames3['dropdown--has-title'] = Boolean(this.props.title), _classNames3['dropdown--has-footer'] = this.props.footerTitle, _classNames3["dropdown--" + this.props.name] = Boolean(this.props.name), _classNames3["dropdown--" + this.props.theme] = true, _classNames3));
    var arrowClassNames = (0, _classnames["default"])((_classNames4 = {
      'dropdown__arrow': true,
      'dropdown__arrow--legacy': true
    }, _classNames4["dropdown__arrow--" + this.props.arrowPosition] = true, _classNames4["dropdown__arrow--" + position] = true, _classNames4["dropdown__arrow--" + position + "-" + this.props.arrowPosition] = true, _classNames4["dropdown__arrow-" + position + "--has-title"] = this.props.title, _classNames4["dropdown__arrow-" + position + "--has-footer"] = this.props.footerTitle, _classNames4["dropdown__arrow-" + position + "-" + this.props.arrowPosition + "--has-title"] = this.props.title, _classNames4["dropdown__arrow-" + position + "-" + this.props.arrowPosition + "--has-footer"] = this.props.footerTitle, _classNames4['dropdown__arrow--has-footer'] = this.props.footerTitle, _classNames4["dropdown__arrow--" + this.props.name] = Boolean(this.props.name), _classNames4["dropdown__arrow--" + this.props.theme] = true, _classNames4));
    var dropdownPopupClassNames = (0, _classnames["default"])((_classNames5 = {
      'dropdown__popup-wrapper': true,
      'dropdown__popup-wrapper--legacy': true
    }, _classNames5["dropdown__popup-wrapper--" + this.props.name] = Boolean(this.props.name), _classNames5["dropdown__popup-wrapper--" + this.props.theme] = true, _classNames5));
    var dropdownBackdropClassNames = (0, _classnames["default"])((_classNames6 = {
      'dropdown__backdrop': true,
      'dropdown__backdrop--legacy': true
    }, _classNames6["dropdown__backdrop--" + this.props.name] = Boolean(this.props.name), _classNames6["dropdown__backdrop--" + this.props.theme] = true, _classNames6));
    var dropdownTitleClassNames = (0, _classnames["default"])((_classNames7 = {
      'dropdown__title': true,
      'dropdown__title--legacy': true
    }, _classNames7["dropdown__title--" + this.props.name] = Boolean(this.props.name), _classNames7["dropdown__title--" + this.props.theme] = true, _classNames7));
    var dropdownFooterClassNames = (0, _classnames["default"])((_classNames8 = {
      'dropdown__footer': true,
      'dropdown__footer--legacy': true
    }, _classNames8["dropdown__footer--" + this.props.name] = Boolean(this.props.name), _classNames8["dropdown__footer--" + this.props.theme] = true, _classNames8));

    if (this.state.isDropdownVisible) {
      var dropdownBackdrop;

      if (!hoverClassName) {
        dropdownBackdrop = /*#__PURE__*/_react["default"].createElement("div", _extends({}, eventHandlers, {
          "data-ui": "dropdown__backdrop",
          className: dropdownBackdropClassNames
        }));
      }

      var title;

      if (this.props.title) {
        title = /*#__PURE__*/_react["default"].createElement("div", {
          className: dropdownTitleClassNames
        }, this.props.title);
      }

      var content = this.props.content || this.props.children;

      if (this.props.options) {
        content = /*#__PURE__*/_react["default"].createElement(_dropdownList["default"], {
          theme: this.props.theme,
          options: this.props.options,
          onRequestHide: this._handleDropdown
        });
      }

      var footer;

      if (this.props.footerTitle) {
        footer = /*#__PURE__*/_react["default"].createElement("button", {
          className: dropdownFooterClassNames,
          onClick: this.props.onFooterClick
        }, this.props.footerTitle);
      }

      dropdown = /*#__PURE__*/_react["default"].createElement("div", {
        className: dropdownPopupClassNames
      }, dropdownBackdrop, /*#__PURE__*/_react["default"].createElement("div", {
        ref: function ref(node) {
          _this3._dropdownWrapper = node;
        },
        "data-ui": "dropdown__wrapper",
        className: dropdownWrapperClassNames,
        style: {
          marginLeft: this.state.dropdownOffset
        }
      }, /*#__PURE__*/_react["default"].createElement("div", {
        ref: function ref(node) {
          _this3._dropdown = node;
        },
        className: dropdownClassNames
      }, /*#__PURE__*/_react["default"].createElement("div", {
        className: arrowClassNames,
        style: {
          marginLeft: this.state.dropdownOffset * -1
        }
      }), title, content, footer)));
    }

    var dropdownEvents = {};

    if (this.props.triggerEvents.includes('hover')) {
      dropdownEvents = {
        onMouseEnter: this._adjustPosition
      };
    }

    return /*#__PURE__*/_react["default"].createElement("div", _extends({
      ref: function ref(node) {
        _this3._dropdownContainer = node;
      },
      className: containerClassNames
    }, dropdownEvents), /*#__PURE__*/_react["default"].cloneElement(this.props.triggerNode, _extends({}, eventHandlers)), dropdown);
  };

  return Dropdown;
}(_react["default"].Component);

exports["default"] = Dropdown;

_defineProperty(Dropdown, "displayName", 'DropdownLegacy');

_defineProperty(Dropdown, "defaultProps", {
  arrowPosition: 'center',
  boundingElement: null,
  triggerEvents: ['onClick'],
  position: 'bottom',
  visible: null,
  autoPosition: false,
  disabled: false,
  name: '',
  theme: 'dark'
});