"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _classnames = _interopRequireDefault(require("classnames"));

var _reactWindowResizeListener = require("react-window-resize-listener");

var _dropdownList = _interopRequireDefault(require("./dropdown-list"));

var _positioningUtils = _interopRequireDefault(require("../../utils/positioning-utils"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Dropdown = /*#__PURE__*/function (_React$PureComponent) {
  _inheritsLoose(Dropdown, _React$PureComponent);

  function Dropdown(_props, context) {
    var _this;

    _this = _React$PureComponent.call(this, _props, context) || this; // NOTE: using classic functions because of issues related to testing using refs

    _defineProperty(_assertThisInitialized(_this), "state", {});

    _defineProperty(_assertThisInitialized(_this), "_getState", function (props) {
      var isDropdownVisible = (_this.props.visible && !_this._isHoverEnabled() || _this.state.isDropdownVisible) && !props.disabled;

      if (!isDropdownVisible || !_this._triggerNodeRef || !_this._dropdownPortalRef) {
        // @ts-ignore
        return {
          isDropdownVisible: isDropdownVisible
        };
      }

      var windowState = _positioningUtils["default"].getWindowState({
        fixedScroll: !props.scroll
      }); // NOTE: dropdown will position itself around this node (either anchor node or default triggerNode)


      var anchorRect = _this._getAnchorRect(props);

      var anchorCenter = _positioningUtils["default"].getElementCenter({
        rect: anchorRect,
        windowState: windowState
      });

      var anchorSize = {
        width: anchorRect.width,
        height: anchorRect.height
      };

      var contentSize = _this._getContentSize();

      var offsetMaybeWithOverflow = _positioningUtils["default"].getOffsetDueToAnchor({
        position: props.position,
        arrowPosition: props.arrowPosition,
        anchorSize: anchorSize,
        contentSize: contentSize
      });

      var overflow = _positioningUtils["default"].calculateOverflow({
        offset: offsetMaybeWithOverflow,
        contentSize: contentSize,
        anchorCenter: anchorCenter,
        windowState: windowState
      });

      var offset = _positioningUtils["default"].adjustOffsetForOverflow({
        position: props.position,
        overflow: overflow,
        offset: offsetMaybeWithOverflow,
        contentSize: contentSize,
        anchorSize: anchorSize
      });

      var overriddenPosition = _positioningUtils["default"].getOverriddenPosition({
        overflow: overflow,
        position: props.position
      });

      var dropdownStyle = {
        top: anchorCenter.y + offset.y,
        left: anchorCenter.x + offset.x
      };
      return {
        arrowOffset: _positioningUtils["default"].isTopOrBottom(props.position) ? -offset.x : -offset.y,
        dropdownStyle: dropdownStyle,
        overriddenPosition: overriddenPosition,
        isDropdownVisible: isDropdownVisible
      };
    });

    _defineProperty(_assertThisInitialized(_this), "_isHoverEnabled", function () {
      return _this.props.triggerEvents === 'hover' || Array.isArray(_this.props.triggerEvents) && _this.props.triggerEvents.includes('hover');
    });

    _defineProperty(_assertThisInitialized(_this), "_getAnchorRect", function (props) {
      if (!props.anchorNode && !_this._triggerNodeRef) {
        return;
      }

      var anchorNode = props.anchorNode || _this._triggerNodeRef;
      return anchorNode.getBoundingClientRect();
    });

    _defineProperty(_assertThisInitialized(_this), "_getContentSize", function () {
      var dropdownContentRect = _this._dropdownPortalRef && _this._dropdownPortalRef.getBoundingClientRect();

      return {
        width: dropdownContentRect ? dropdownContentRect.width : 0,
        height: dropdownContentRect ? dropdownContentRect.height : 0
      };
    });

    _defineProperty(_assertThisInitialized(_this), "_updateArrowCssVariable", function (position) {
      if (!_this._dropdownPortalRef) {
        return;
      } // NOTE: arrow is rendered using ':before' pseudoclass. React cannot modify it dynamically.


      if (_positioningUtils["default"].isTopOrBottom(position)) {
        _this._dropdownPortalRef.style.setProperty('--js-dropdown-left', _this.state.arrowOffset + "px");
      } else {
        _this._dropdownPortalRef.style.setProperty('--js-dropdown-top', _this.state.arrowOffset + "px");
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_handleDropdown", function () {
      if (_this.props.visible !== null && _this.props.onVisibilityChange) {
        _this.props.onVisibilityChange(!_this.state.isDropdownVisible);
      } else {
        _this.setState({
          isDropdownVisible: !_this.state.isDropdownVisible
        });
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_handleWindowResize", function () {
      if (_this.state.isDropdownVisible) {
        _this.setState(_this._getState(_this.props));
      }
    });

    _defineProperty(_assertThisInitialized(_this), "_handleMouseEnter", function () {
      _this.setState({
        isDropdownVisible: true
      });
    });

    _defineProperty(_assertThisInitialized(_this), "_handleMouseLeave", function () {
      _this.setState({
        isDropdownVisible: false
      });
    });

    _this._setDropdownPortalRef = _this._setDropdownPortalRef.bind(_assertThisInitialized(_this));
    _this._setTriggerNodeRef = _this._setTriggerNodeRef.bind(_assertThisInitialized(_this));
    _this.state = _this._getState(_this.props);
    return _this;
  } // @ts-ignore


  var _proto = Dropdown.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.setState(this._getState(this.props));
  };

  _proto.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    if (this.props.triggerEvents !== nextProps.triggerEvents || this.props.disabled !== nextProps.disabled || this.props.visible !== nextProps.visible) {
      this.setState({
        isDropdownVisible: !nextProps.disabled && nextProps.visible && this._isOnClickEnabled(nextProps)
      });
    }
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    if (!prevState.isDropdownVisible && this.state.isDropdownVisible) {
      // NOTE: we have to update state here because we can't determine dropdown size before it's rendered.
      // eslint-disable-next-line react/no-did-update-set-state
      this.setState(this._getState(this.props));
    } // @ts-ignore


    if (!prevState.arrowOffset !== this.state.arrowOffset) {
      this._updateArrowCssVariable(this.props.position);
    }
  };

  _proto._isOnClickEnabled = function _isOnClickEnabled(props) {
    if (props === void 0) {
      props = this.props;
    }

    return props.triggerEvents === 'onClick' || Array.isArray(props.triggerEvents) && props.triggerEvents.includes('onClick');
  };

  _proto._createEventHandlers = function _createEventHandlers() {
    var _this2 = this;

    var eventHandlers = {};

    if (this.props.triggerEvents && Array.isArray(this.props.triggerEvents)) {
      this.props.triggerEvents.forEach(function (event) {
        eventHandlers[event] = _this2._handleDropdown;
      });
    } else {
      // @ts-ignore
      eventHandlers[this.props.triggerEvents] = this._handleDropdown;
    }

    delete eventHandlers['hover'];
    return eventHandlers;
  };

  _proto._setDropdownPortalRef = function _setDropdownPortalRef(node) {
    this._dropdownPortalRef = node;
  };

  _proto._setTriggerNodeRef = function _setTriggerNodeRef(node) {
    this._triggerNodeRef = node;
  };

  _proto.render = function render() {
    var _classNames, _classNames2, _classNames3, _classNames4, _classNames5, _classNames6, _classNames7;

    var eventHandlers = this._createEventHandlers();

    var dropdown = /*#__PURE__*/_react["default"].createElement("div", null);

    var position = this.state.overriddenPosition || this.props.position;
    var hoverClassName = false;

    if (Array.isArray(this.props.triggerEvents)) {
      hoverClassName = this.props.triggerEvents.indexOf('hover') !== -1;
    } else if (this.props.triggerEvents === 'hover') {
      hoverClassName = true;
    }

    var containerClassNames = (0, _classnames["default"])(this.props.containerClassNames, (_classNames = {
      'dropdown-portal-container': true,
      'dropdown-portal-container--hover': hoverClassName
    }, _classNames["dropdown-portal-container--" + this.props.name] = Boolean(this.props.name), _classNames["dropdown-portal-container--" + this.props.theme] = true, _classNames));
    var dropdownWrapperClassNames = (0, _classnames["default"])((_classNames2 = {
      'dropdown-portal__wrapper': true,
      'dropdown-portal__wrapper--portal': true
    }, _classNames2["dropdown-portal__wrapper--" + position] = true, _classNames2["dropdown-portal__wrapper--" + this.props.arrowPosition] = true, _classNames2["dropdown-portal__wrapper--" + position + "-" + this.props.arrowPosition] = true, _classNames2['dropdown-portal__wrapper--visible'] = this.state.isDropdownVisible, _classNames2['dropdown-portal__wrapper--scroll'] = this.props.scroll, _classNames2['dropdown-portal__wrapper--fixed'] = !this.props.scroll, _classNames2['dropdown-portal__wrapper--hover'] = hoverClassName, _classNames2["dropdown-portal__wrapper--" + this.props.name] = Boolean(this.props.name), _classNames2["dropdown-portal__wrapper--" + this.props.theme] = true, _classNames2));
    var dropdownClassNames = (0, _classnames["default"])(this.props.className, (_classNames3 = {
      'dropdown-portal': true
    }, _classNames3["dropdown-portal--" + position] = true, _classNames3['dropdown-portal--visible'] = this.state.isDropdownVisible, _classNames3['dropdown-portal--with-arrow'] = typeof this.props.showArrow === 'undefined' ? true : this.props.showArrow, _classNames3['dropdown-portal--has-title'] = Boolean(this.props.title), _classNames3['dropdown-portal--has-footer'] = this.props.footerTitle, _classNames3["dropdown-portal--" + this.props.name] = Boolean(this.props.name), _classNames3["dropdown-portal--" + this.props.theme] = true, _classNames3));
    var dropdownPortalClassNames = (0, _classnames["default"])((_classNames4 = {
      'dropdown-portal-wrapper': true,
      'dropdown-portal-wrapper--portal': true
    }, _classNames4["dropdown-portal-wrapper--" + this.props.name] = Boolean(this.props.name), _classNames4["dropdown-portal-wrapper--" + this.props.theme] = true, _classNames4));
    var dropdownBackdropClassNames = (0, _classnames["default"])((_classNames5 = {
      'dropdown-portal__backdrop': true,
      'dropdown-portal__backdrop--portal': true
    }, _classNames5["dropdown-portal__backdrop--" + this.props.name] = Boolean(this.props.name), _classNames5["dropdown-portal__backdrop--" + this.props.theme] = true, _classNames5));
    var dropdownTitleClassNames = (0, _classnames["default"])((_classNames6 = {
      'dropdown-portal__title': true,
      'dropdown-portal__title--portal': true
    }, _classNames6["dropdown-portal__title--" + this.props.name] = Boolean(this.props.name), _classNames6["dropdown-portal__title--" + this.props.theme] = true, _classNames6));
    var dropdownFooterClassNames = (0, _classnames["default"])((_classNames7 = {
      'dropdown-portal__footer': true,
      'dropdown-portal__footer--portal': true
    }, _classNames7["dropdown-portal__footer--" + this.props.name] = Boolean(this.props.name), _classNames7["dropdown-portal__footer--" + this.props.theme] = true, _classNames7));
    var dropdownEvents = {};

    if (this.props.triggerEvents.includes('hover')) {
      dropdownEvents = {
        onMouseEnter: this._handleMouseEnter,
        onMouseLeave: this._handleMouseLeave
      };
    }

    if (this.state.isDropdownVisible) {
      var dropdownBackdrop;

      if (!hoverClassName) {
        dropdownBackdrop = /*#__PURE__*/_react["default"].createElement("div", _extends({}, eventHandlers, {
          "data-ui": "dropdown-portal__backdrop",
          className: dropdownBackdropClassNames
        }));
      }

      var title;

      if (this.props.title) {
        title = /*#__PURE__*/_react["default"].createElement("div", {
          className: dropdownTitleClassNames
        }, this.props.title);
      }

      var content = this.props.content || this.props.children;

      if (this.props.options) {
        content = /*#__PURE__*/_react["default"].createElement(_dropdownList["default"], {
          theme: this.props.theme,
          options: this.props.options,
          onRequestHide: this._handleDropdown
        });
      }

      var footer;

      if (this.props.footerTitle) {
        footer = /*#__PURE__*/_react["default"].createElement("button", {
          className: dropdownFooterClassNames,
          onClick: this.props.onFooterClick
        }, this.props.footerTitle);
      }

      dropdown = /*#__PURE__*/_react["default"].createElement("div", {
        className: dropdownPortalClassNames
      }, dropdownBackdrop, /*#__PURE__*/_react["default"].createElement("div", {
        "data-ui": "dropdown-portal__wrapper",
        className: dropdownWrapperClassNames,
        style: this.state.dropdownStyle
      }, /*#__PURE__*/_react["default"].createElement("div", {
        ref: this._setDropdownPortalRef,
        className: dropdownClassNames
      }, title, content, footer)));
    }

    return /*#__PURE__*/_react["default"].createElement("div", _extends({
      ref: this._setTriggerNodeRef,
      className: containerClassNames
    }, dropdownEvents), /*#__PURE__*/_react["default"].cloneElement(this.props.triggerNode, _extends({}, eventHandlers)), /*#__PURE__*/_reactDom["default"].createPortal(dropdown, this.props.portalNode || document.body), /*#__PURE__*/_react["default"].createElement(_reactWindowResizeListener.WindowResizeListener, {
      onResize: this._handleWindowResize
    }));
  };

  return Dropdown;
}(_react["default"].PureComponent);

exports["default"] = Dropdown;

_defineProperty(Dropdown, "displayName", 'Dropdown');

_defineProperty(Dropdown, "defaultProps", {
  arrowPosition: 'center',
  triggerEvents: ['onClick'],
  position: 'bottom',
  visible: null,
  disabled: false,
  name: '',
  scroll: true,
  theme: 'dark'
});