"use strict";

exports.__esModule = true;
exports["default"] = exports.DISTANCE_BETWEEN_EDGE_AND_ARROW = void 0;

/* eslint-disable no-mixed-operators */
var DISTANCE_BETWEEN_EDGE_AND_ARROW = 20;
exports.DISTANCE_BETWEEN_EDGE_AND_ARROW = DISTANCE_BETWEEN_EDGE_AND_ARROW;

var PositioningUtils = /*#__PURE__*/function () {
  function PositioningUtils() {}

  PositioningUtils.isLeftOrRight = function isLeftOrRight(position) {
    return position === 'left' || position === 'right';
  };

  PositioningUtils.isTopOrBottom = function isTopOrBottom(position) {
    return position === 'top' || position === 'bottom';
  };

  PositioningUtils.getWindowState = function getWindowState(options) {
    if (options === void 0) {
      options = {};
    }

    return {
      scroll: {
        x: options.fixedScroll ? 0 : window.scrollX,
        y: options.fixedScroll ? 0 : window.scrollY
      },
      size: {
        width: document.documentElement.clientWidth,
        height: document.documentElement.clientHeight
      }
    };
  };

  PositioningUtils.getElementCenter = function getElementCenter(_temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        rect = _ref.rect,
        windowState = _ref.windowState;

    return {
      x: windowState.scroll.x + rect.left + rect.width / 2,
      y: windowState.scroll.y + rect.top + rect.height / 2
    };
  };

  PositioningUtils.getOffsetDueToAnchor = function getOffsetDueToAnchor(_temp2) {
    var _ref2 = _temp2 === void 0 ? {} : _temp2,
        position = _ref2.position,
        arrowPosition = _ref2.arrowPosition,
        anchorSize = _ref2.anchorSize,
        contentSize = _ref2.contentSize;

    // NOTE: returns relative coordinates with respect to the center of the anchor node
    var offset = PositioningUtils.getOffsetDueToAnchorWithoutArrowPosition({
      position: position,
      anchorSize: anchorSize,
      contentSize: contentSize
    });
    var extraOffsetByArrowPosition = PositioningUtils.getExtraOffsetByArrowPosition({
      arrowPosition: arrowPosition,
      position: position,
      contentSize: contentSize
    });
    return {
      x: offset.x + extraOffsetByArrowPosition.x,
      y: offset.y + extraOffsetByArrowPosition.y
    };
  };

  PositioningUtils.getOffsetDueToAnchorWithoutArrowPosition = function getOffsetDueToAnchorWithoutArrowPosition(_temp3) {
    var _ref3 = _temp3 === void 0 ? {} : _temp3,
        position = _ref3.position,
        contentSize = _ref3.contentSize,
        anchorSize = _ref3.anchorSize;

    switch (position) {
      case 'top':
        return {
          // NOTE: anchor center X matches dropdown center X.
          x: -contentSize.width / 2,
          // NOTE: dropdown's bottom matches anchor's top.
          y: -(anchorSize.height / 2) - contentSize.height
        };

      case 'right':
        return {
          x: anchorSize.width / 2,
          y: -contentSize.height / 2
        };

      case 'bottom':
        return {
          x: -contentSize.width / 2,
          y: anchorSize.height / 2
        };

      case 'left':
        return {
          x: -anchorSize.width / 2 - contentSize.width,
          y: -contentSize.height / 2
        };
    }
  };

  PositioningUtils.getExtraOffsetByArrowPosition = function getExtraOffsetByArrowPosition(_temp4) {
    var _ref4 = _temp4 === void 0 ? {} : _temp4,
        position = _ref4.position,
        contentSize = _ref4.contentSize,
        arrowPosition = _ref4.arrowPosition;

    var halfSize = PositioningUtils.isTopOrBottom(position) ? contentSize.width / 2 : contentSize.height / 2;

    if (PositioningUtils.isTopOrBottom(position)) {
      if (arrowPosition === 'start') {
        return {
          x: halfSize - DISTANCE_BETWEEN_EDGE_AND_ARROW,
          y: 0
        };
      }

      if (arrowPosition === 'end') {
        return {
          x: -halfSize + DISTANCE_BETWEEN_EDGE_AND_ARROW,
          y: 0
        };
      }
    }

    if (PositioningUtils.isLeftOrRight(position)) {
      if (arrowPosition === 'start') {
        return {
          x: 0,
          y: halfSize - DISTANCE_BETWEEN_EDGE_AND_ARROW
        };
      }

      if (arrowPosition === 'end') {
        return {
          x: 0,
          y: -halfSize + DISTANCE_BETWEEN_EDGE_AND_ARROW
        };
      }
    }

    return {
      x: 0,
      y: 0
    };
  };

  PositioningUtils.calculateOverflow = function calculateOverflow(_temp5) {
    var _ref5 = _temp5 === void 0 ? {} : _temp5,
        offset = _ref5.offset,
        contentSize = _ref5.contentSize,
        anchorCenter = _ref5.anchorCenter,
        windowState = _ref5.windowState;

    var windowScroll = windowState.scroll;
    var windowSize = windowState.size; // NOTE: tells by how many pixels dropdown is out of bounds.
    // if positive, then out of bounds. If negative, no action necessary.
    // two overflows are possible at the same time (e.g. top and right)

    var right = anchorCenter.x + offset.x + contentSize.width - (windowSize.width + windowScroll.x);
    var left = windowScroll.x - (anchorCenter.x + offset.x);
    var top = windowScroll.y - (anchorCenter.y + offset.y);
    var bottom = anchorCenter.y + offset.y + contentSize.height - (windowSize.height + windowScroll.y);
    return {
      top: top,
      right: right,
      bottom: bottom,
      left: left
    };
  };

  PositioningUtils.adjustOffsetForOverflow = function adjustOffsetForOverflow(_temp6) {
    var _ref6 = _temp6 === void 0 ? {} : _temp6,
        overflow = _ref6.overflow,
        offset = _ref6.offset,
        contentSize = _ref6.contentSize,
        anchorSize = _ref6.anchorSize,
        position = _ref6.position;

    if (PositioningUtils.isTopOrBottom(position)) {
      // NOTE: either top or bottom is zero
      var _top = overflow.top > 0 ? contentSize.height + anchorSize.height : 0; // NOTE: opposite sign of "top"


      var _bottom = overflow.bottom > 0 ? -(contentSize.height + anchorSize.height) : 0; // NOTE: either left or right is zero


      var _left = overflow.left > 0 ? overflow.left : 0;

      var _right = overflow.right > 0 ? -overflow.right : 0;

      return {
        x: offset.x + _left + _right,
        y: offset.y + _top + _bottom
      };
    }

    var top = overflow.top > 0 ? overflow.top : 0;
    var bottom = overflow.bottom > 0 ? -overflow.bottom : 0;
    var left = overflow.left > 0 ? contentSize.width + anchorSize.width : 0;
    var right = overflow.right > 0 ? -(contentSize.width + anchorSize.width) : 0;
    return {
      x: offset.x + left + right,
      y: offset.y + top + bottom
    };
  };

  PositioningUtils.getOverriddenPosition = function getOverriddenPosition(_ref7) {
    var overflow = _ref7.overflow,
        position = _ref7.position;

    // NOTE: it affects only cosmetic styling (arrow position), but not the position (top, left) itself.
    if (PositioningUtils.isTopOrBottom(position) && overflow.top > 0) {
      return 'bottom';
    }

    if (PositioningUtils.isTopOrBottom(position) && overflow.bottom > 0) {
      return 'top';
    }

    if (PositioningUtils.isLeftOrRight(position) && overflow.left > 0) {
      return 'right';
    }

    if (PositioningUtils.isLeftOrRight(position) && overflow.right > 0) {
      return 'left';
    }

    return null;
  };

  return PositioningUtils;
}();

exports["default"] = PositioningUtils;